# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : model.py 
# @Software: Visual Studio Code

from sqlalchemy import TIMESTAMP, Column, Integer, SmallInteger, String, ForeignKey, Text, Sequence, UniqueConstraint, func
from sqlalchemy.orm import declarative_base

Base = declarative_base()

class TimeMixedIn:
    create_time = Column(TIMESTAMP(timezone=True), default=func.now(), comment='创建时间')
    modify_time = Column(TIMESTAMP(timezone=True), default=func.now(), onupdate=func.now(), comment='修改时间')


class User(Base, TimeMixedIn):
    __tablename__ = 'user'

    id = Column(Integer, Sequence('user_id_seq'), primary_key=True)
    user_id = Column(String(32), nullable=False, unique=True)
    user_ref = Column(String(128), nullable=False, unique=True)
    user_name = Column(String(64), nullable=True)
    lang_code = Column(String(16), nullable=False)
    referral_code = Column(String(32), nullable=False, unique=True, comment="referral code")
    referraled_code = Column(String(32), nullable=True, comment="referraled code")

class Video(Base, TimeMixedIn):
    __tablename__ = 'video'

    id = Column(Integer, primary_key=True)
    name = Column(String(128), nullable=False, default='', comment='展示给用户的名称')
    filename = Column(String(128), nullable=False, default='', comment='文件名')
    bangou = Column(String(64), nullable=False, unique=True, default='', comment='番号名')

    preview_url = Column(Text, nullable=False, default='')
    online_url = Column(Text, nullable=False, default='')
    magnet_url = Column(Text, nullable=False, default='')

    md5 = Column(String(64), nullable=False, unique=True)
    object_key = Column(String(128), nullable=False, unique=True)

    price = Column(SmallInteger, nullable=False, default=1)
    star = Column(SmallInteger, nullable=False, default=0)
    status = Column(SmallInteger, nullable=False, default=0, comment="0: draft, 1: final")

    def __repr__(self):
        return f"<Video(id='{self.id}', name='{self.name}', bangou='{self.bangou}')>"

class VideoTags(Base, TimeMixedIn):
    __tablename__ = 'video_tags'

    id = Column(Integer, Sequence('video_tags_id_seq'), primary_key=True)
    video_id = Column(Integer, ForeignKey('video.id'))
    tag = Column(String(16), nullable=False)

class Message(Base, TimeMixedIn):
    __tablename__ = 'message'

    id = Column(Integer, Sequence('message_id_seq'), primary_key=True)
    message_id = Column(Integer, nullable=False, comment="telegram message id")
    edit_message_id = Column(Integer, nullable=False, default=0)
    from_id = Column(String(32), nullable=False)
    send_id = Column(String(32), nullable=False)
    content = Column(Text, nullable=False, default='')
    button = Column(Text, nullable=False, default='')
    timestamp = Column(TIMESTAMP(timezone=True), nullable=False)

class Payment(Base, TimeMixedIn):

    __tablename__ = 'payment'

    id = Column(Integer, Sequence('payment_id_seq'), primary_key=True)
    user_id = Column(String(32), nullable=False, unique=True)
    amount = Column(Integer, nullable=False, default=0)

class Redeem(Base, TimeMixedIn):

    __tablename__ = 'redeem'

    id = Column(Integer, Sequence('redeem_id_seq'), primary_key=True)
    user_id = Column(String(32), nullable=False, unique=True)
    gift_code = Column(String(32) , nullable=False)


class Purchase(Base, TimeMixedIn):

    __tablename__ = 'purchase'

    id = Column(Integer, Sequence('purchase_id_seq'), primary_key=True)
    user_id = Column(String(32), nullable=False, unique=True)
    video_id = Column(Integer, ForeignKey('video.id'))